#!/bin/sh


DIR_TMP="/boot"

BUILD="ukmcli-build"
BUILD_ARCHIVE="$BUILD.tgz"
ROOT="ukm-root"
ROOT_ARCHIVE="$ROOT.tar.gz"

LOCAL='/usr/local/'
MYSQLDIR=$LOCAL'mysql/'

TMP_DIR="/tmp/mysql_create/"

DIRS_2_REMOVE="$TMP_DIR"

DBGRANTS_SQL=${TMP_DIR}grants.sql
DB_SQL=${TMP_DIR}db.sql
DB_NAME="ukmclient"


if [ ${#1} -gt 0 ]; then
  DB_NAME=$1
fi

SERVICE=$DB_NAME
RC_UKM=$LOCAL$DB_NAME'/rc.ukm'


ADMIN_DB_USER='root'
granted_db_user='ukm_terminal'
password_for_granted_db_user='CtHDbCGK.C'


#####################################
# Functions begin

lsdir()
{
	local dir=$1

	if [ -d $dir ]; then
		echo
		echo "---- $dir content"
		ls -lRa $dir
		echo "---- $dir content"
		echo
	else
		echo "lsdir($dir): directory \"$dir\" not found"
	fi
}


rollback_on_fault()
{
	echo
	echo
	echo "==== rollback_on_fault() begin"
	
	echo "BACKUP_IS_READY=$BACKUP_IS_READY"
	
	if [ "$BACKUP_IS_READY" == "Y" ]; then
		echo
		lsdir $DIR_BOOT
		echo -n "Checking dir $DIR_BACKUP ... "
		if [ -d $DIR_BACKUP ]; then
			echo "found"
			echo
			echo "Removing $DIR_BOOT content"
			rm -fv $DIR_BOOT/*
			echo
			
			echo "Restoring boot files"
			cp -fv $DIR_BACKUP/boot/* $DIR_BOOT
			echo
			
			lsdir $DIR_BOOT
		else
			echo "not found. Nothing to restore"
		fi
		
		echo -n "Checking dir $DIR_K_MODULES ... "
		if [ -d $DIR_K_MODULES ]; then
			echo "found"
			echo "Removing directory $DIR_K_MODULES"
			rm -frv $DIR_K_MODULES
		else
			echo "not found"
		fi
	else
		echo "Backup is not ready - nothing can be done"
	fi
	
	echo "==== rollback_on_fault() end"
}

result()
{
	if [ $? -ne 0 ]; then
		echo "ERROR"
#		rollback_on_fault
		echo
		echo "$0 failed"
		echo
		exit 1
	fi
}

sleepv()
{
	local secs_to_sleep=$1
	echo "Sleeping for $secs_to_sleep sec ..."
	sleep $secs_to_sleep
}

dir_create()
{
	local dir=$1
	
	echo "dir_create() checking argument 1 ($dir)"
	[ -n "$dir" ]
	result
	
	if [ ! -d $dir ]; then
		echo "dir_create($dir) - creating directory"
		mkdir -pv $dir
		result
	fi
}

dir_recreate()
{
	local dir=$1
	echo "dir_recreate($dir) checking argument 1 ($dir)"
	[ -n "$dir" ]
	result
	
	if [ -d $dir ]; then
		echo "dir_recreate(): Directory $dir exists - removing it"
		rm -fr $dir
		result
	fi
	
	dir_create $dir
}

catv()
{
	local fname=$1
	echo
	echo "==== $fname ============"
	cat $fname
	echo "==== $fname ============"
	echo
}

cpv()
{
	local src=$1
	local dst=$2
	
	echo
	echo "cpv checking argument src ($src)"
	[ -n "$src" ]
	result
	
	echo "cpv checking argument dst ($dst)"
	[ -n "$dst" ]
	result
	
	echo "cpv(): Copying $src to $dst"
	cp $CP_OPTIONS $src $dst
	result
}


cp_dir_content()
{
	local src=$1
	local dst=$2
	echo
	echo
	echo "-- cp_dir_content() begin"
	echo "Copying $src content to $dst"
	cp $CP_OPTIONS $src/* $dst
	result
	echo "-- cp_dir_content() begin"
	echo
	echo
}


mvv()
{
	local src=$1
	local dst=$2
	
	echo "mvv(): Moving \"$src\" to \"$dst\""
	mv -fv $src $dst
	result
}

unpackto()
{
	local file_to_unpack=$1
	local dir_unpack_to=$2
	local msg="-- Unpacking \"$file_to_unpack\""

	echo "unpackto() checking 1st parameter"
	[ -n "$file_to_unpack" ]
	result

	echo "unpackto() checking archive file $file_to_unpack"
	[ -f $file_to_unpack ]
	result

	if [ -n "$dir_unpack_to" ]; then
		dir_create $dir_unpack_to
		local tar_params="--directory $dir_unpack_to"
		msg="$msg to \"$dir_unpack_to\" started"
	else
	 msg="$msg to \"$(pwd)\" started"
	fi

	if [ ! -d $dir_unpack_to ]; then
		dir_create $dir_unpack_to
	fi

	echo
	echo $msg

	tar xvzf $file_to_unpack $tar_params
	result
	echo "-- Unpacking of $file_to_unpack completed"
	echo
	echo
}

cdv()
{
	local dstdir=$1
	
	echo "cdv() checking Directory \"$dstdrir\""
	[ -d $dstdir ]
	result
	
	echo "dst() changing current directory to \"$dstdir\""
	cd $dstdir
	result
	echo "Current directory is set to $(pwd)"
}

packdir()
{
	local path_to_pack=$1
	local archive_name=$2
	
	echo "packdir(): path_to_pack=\"$path_to_pack\" archive_name=\"$archive_name\""
	echo "packdir(): checking parameter path_to_pack"
	[ -n "$path_to_pack" ]
	result
	
	echo "packdir(): checking directory $path_to_pack"
	[ -d $path_to_pack ]
	result
	
	echo "packdir(): checking parameter archive_name"
	[ -n "$archive_name" ]
	result
	
	local archive_name_parent_dir=`dirname $archive_name`
	if [ -n "$archive_name_parent_dir" ]; then
		if [ ! -d "$archive_name_parent_dir" ]; then
			echo "Warning: packdir() archive parent directory \"$archive_name_parent_dir\" not found. Trying to create"
			dir_create $archive_name_parent_dir
		fi
	fi

	local dir_start=`pwd`

	local dir_to_pack_parent=`dirname $path_to_pack`
	cdv $dir_to_pack_parent

	local dir_to_pack=`basename $path_to_pack`
	echo "Compressing $dir_to_pack to $archive_name"
	tar czf $archive_name $dir_to_pack
	result

	cdv $dir_start
}


rmv()
{
	local remove_what=$1

	if [ -z "$remove_what" ]; then
		echo "Warning: rmv() 1st parameter is empty"
		return 0
	fi

	echo "Removing $remove_what"
	rm -frv $remove_what
	result
}

check_file()
{
	local what_to_check=$1
	echo -n "Checking $what_to_check ... "
	[ -f $what_to_check ]
	result
	
	echo "found"
}


################ PROCESS UTILITIES ############

pslist()
{
	local CMD_PS='ps -e --forest -o pid,user,euser,group,egroup,tty,cmd --width=2000'
	echo
	echo
	echo "=========  Active processes list"
	$CMD_PS
	echo "=========  Active processes list"
	echo
	echo
}

getpids()
{
	local process_name=$*
	pids=
	echo -n "Searching pid for \"$process_name\" ... "
	pids=`ps -e -o pid,cmd --width=2000 | grep "$process_name" | grep -v grep | awk -F" " '{ print $1 }'`
	if [ -n "$pids" ]; then
		echo "FOUND"
		return 0
	else
		echo "NOT FOUND"
		return 1
	fi
}

kill_pids()
{
	local process_ids=$*
	local DELAY_AFTER_KILL=5
	if [ -n "$process_ids" ]; then
		echo "$process_ids" | while read pid; do
			echo -n "Killing pid=$pid ... "
			kill -9 $pid > /dev/null 2>&1
			if [ $? -eq 0 ]; then
				echo "killed"
				sleepv $DELAY_AFTER_KILL
			else
				echo "not killed"
			fi
		done
	fi
	echo
	echo
}


killps()
{
	echo
	echo
	echo
	echo "-- killps() begin"
	local ps_name_to_kill=$*
	local TIMEOUT_AFTER_KILL_SEC=15
	
	echo "ps_name_to_kill=\"$ps_name_to_kill\""
	if [ -z "$ps_name_to_kill" ]; then
		echo
		echo "WARNING: killps() ps_name_to_kill is empty"
		return 0
	fi
	pslist
	
	getpids "$ps_name_to_kill"
	if [ $? -eq 0 ]; then
		echo "Attempt 1: Killing $ps_name_to_kill politely"
		killall $ps_name_to_kill
		sleepv $TIMEOUT_AFTER_KILL_SEC
		
		getpids "$ps_name_to_kill"
		if [ $? -eq 0 ]; then
			pslist
			echo "$ps_name_to_kill is active after attempt 1"
			echo "Attempt 2: Killing $ps_name_to_kill with the force"
			kill_pids "$pids"
			sleepv $TIMEOUT_AFTER_KILL_SEC
			
			getpids "$ps_name_to_kill"
			if [ $? -eq 0 ]; then
				pslist
				echo "$ps_name_to_kill is active after attempt 2"
				
				echo "Attempt 3: Killing $ps_name_to_kill with the force"
				kill_pids $pids
				sleepv $TIMEOUT_AFTER_KILL_SEC
				
				getpids "$ps_name_to_kill"
				if [ $? -eq 0 ]; then
					pslist
					echo "======================="
					echo "ERROR: $ps_name_to_kill is still active after 3 attempts to kill it"
					echo "Stopping execution"
					echo "======================="
					cd $DIR_START
					echo
					return 1
				fi
			fi
		fi
		echo
		echo "Process \"$ps_name_to_kill\" terminated"
		echo
		pslist
	fi
	echo
	echo "-- killps() end"
	echo
	echo
}


killpslist()
{
	echo
	echo "-- killpslist() begin"
	local processes_to_kill="$@"
	
	for p in $processes_to_kill; do
		killps $p
		if [ $? -ne 0 ]; then
			echo "ERROR: pskill() failed for \"p\""
			return 1
		fi
	done
	echo "-- killpslist() end"
	echo
	return 0
}

################ PROCESS UTILITIES ############

mksymlink()
{
	local targetname=$1
	local linkname=$2
	echo "Creating symlink $linkname->$targetname"
	ln -sfv $targetname $linkname
	result
}

check_kernel_version()
{
	local current_kver=`uname -r`
	echo "Current kernel version is $current_kver"
	
	if [[ "$current_kver" < "$K_VERSION_MIN" ]]; then
		echo "Min kernel version should be $K_VERSION_MIN"
		echo "Skipping kernel upgrade"
		exit 1
	fi
	
	if [[ "$current_kver" < "$K_VERSION_NEW" ]]; then
		echo "Proceeding upgrade to kernel $K_VERSION_NEW"
	else
		echo "Kernel is already up to date"
		echo "Skipping kernel upgrade"
		exit 1
	fi
}

do_backup()
{
	echo
	echo "== do_backup() started"
	echo
	if [ -d $MYSQLDIR/var ]; then
		echo "found $MYSQLDIR/var"
		mv ${MYSQLDIR}var ${MYSQLDIR}var_bad`ls $MYSQLDIR|awk 'BEGIN {max=0}{if ($0 ~ /var_bad/) {sub(/var_bad/,"");if ($1>max) {max=$1}}} END {print max+1}'`
	else
		echo "not found $MYSQLDIR/var"
	fi
	
	echo
	echo "== do_backup() completed"
	echo
}

remove_old_data()
{
	echo
	echo "== remove_old_data() started"
	echo
	
	for file_2_remove in $FILES_2_REMOVE; do
		rmv $file_2_remove
	done
	
	for dir_2_remove in $DIRS_2_REMOVE; do
		rmv $dir_2_remove
	done
	
	echo
	echo "== remove_old_data() completed"
	echo
}

install_new_files()
{
	echo
	echo "== install_new_files() started"
	echo
	download $BUILD_ARCHIVE
	download $ROOT_ARCHIVE
	
	unpackto $TMP_DIR$BUILD_ARCHIVE $TMP_DIR$BUILD
	unpackto $TMP_DIR$ROOT_ARCHIVE $TMP_DIR$ROOT

	load_mysql
	echo
	echo "== install_new_files() completed"
	echo
}

check_files()
{
	echo
	echo "== check_files() started"
	for file_to_check in $FILES_TO_CHECK; do
		check_file $file_to_check
	done
	echo "== check_files() completed"
	
}


download()
{
	local file=$1
	local link=http://$SERVER/ukminstall/$file
	echo "Starting download $link to $TMP_DIR"
	echo
	wget -P $TMP_DIR $link
	result
	check_file $TMP_DIR$file
}


load_dump_mysql()
{
	local dump=$1
	local mysql_options=$2
	check_file $dump
	echo
	echo "load to mysql $dump with options $mysql_options"
	mysql $mysql_options < $dump
	result
	echo "load $dump success"
}

create_database()
{
local db=$1
[ -n "$db" ]
result
cat > $DB_SQL << EOF
CREATE DATABASE \`$db\` CHARACTER SET 'utf8' COLLATE 'utf8_general_ci';
EOF
}

create_grant()
{
cat > $DBGRANTS_SQL << EOF
delete from mysql.db;
delete from mysql.user;
GRANT ALL ON *.* TO $ADMIN_DB_USER@'localhost' IDENTIFIED BY '$password_for_granted_db_user' WITH GRANT OPTION;
GRANT ALL ON *.* TO $granted_db_user@'localhost' IDENTIFIED BY '$password_for_granted_db_user' WITH MAX_USER_CONNECTIONS 100 GRANT OPTION;
GRANT ALL ON *.* TO ukm_web@'localhost' IDENTIFIED BY 'CtHDbCGK.C' WITH MAX_USER_CONNECTIONS 50;

FLUSH PRIVILEGES;
EOF
#check_file $DBGRANTS_SQL
}

load_mysql()
{
	echo
	echo "Create new DB"
	CP_OPTIONS='-R'
	cpv $TMP_DIR$ROOT/usr/local/mysql/var/ ${MYSQLDIR}
	service_start mysql

	echo "create database (DBNAME=$DB_NAME) ... "
	create_database $DB_NAME
	load_dump_mysql "$DB_SQL" "--user=$ADMIN_DB_USER"

	echo "create tables"
	for FILE in `find $TMP_DIR$BUILD -type f -name "*.sql"|sort -r`; do
	load_dump_mysql "$FILE" "--user=$ADMIN_DB_USER $DB_NAME"
	done
	echo "create tables success"
	echo
	
	echo "load grant to mysql"
	create_grant
	load_dump_mysql "$DBGRANTS_SQL" "--user=$ADMIN_DB_USER"
	echo "grant success"

	
}



service_start()
{
	local=$1
	echo
	echo "Starting service $1"
	echo
	service $1 start
	result
	echo "service $1 started"
	echo
}
service_stop()
{
	local=$1
	echo
	echo "Stop service $1"
	echo
	service $1 stop
	result
	echo "service $1 stoped"
	echo
}


#            Functions end
#########################################


echo
echo "======== $(date) $0 started  "

. $RC_UKM

SERVER=$server
service_stop $SERVICE
service_stop "mysql"
do_backup
dir_recreate $TMP_DIR
install_new_files
rmv $TMP_DIR

echo
echo
echo -e "\\033[0;33m"
echo "ЗАБЛОКИРУЙТЕ КАССУ НА СЕРВЕРЕ И ВВЕДИТЕ НА КАССЕ ЕЕ НОМЕР"
echo "ПРИ ВОЗНИКНОВЕНИИ ОШИБКИ НА КАССЕ, ПРОДОЛЖАЕМ РАБОТУ!!! "
echo -e "\\033[0m"
echo
echo

echo "======== $(date) SUCCESS: $0 completed"
echo
service_start $SERVICE


exit 0